const express = require('express');

const idhmrApp = express.Router();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const squel = require('squel');

const query = require(`${libs}/middlewares/query`);

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const id2str = require(`${libs}/middlewares/id2str`);

let rqf = new ReqQueryFields();

idhmrApp.get('/year_range', (req, res, next) => {
    req.sql.from('adh_idh')
    .field('MIN(adh_idh.ano_censo)', 'start_year')
    .field('MAX(adh_idh.ano_censo)', 'end_year');
    next();
}, query, (req, res, next) => {
    req.sql.from('adh_idh_uf')
    .field('MIN(adh_idh_uf.ano_censo)', 'start_year')
    .field('MAX(adh_idh_uf.ano_censo)', 'end_year');
    req.old_result = req.result;
    next();
}, query, (req, res, next) => {
    // console.log(req.old_result[0].start_year);
    // console.log(req.result[0].start_year);
    if (req.old_result[0].start_year < req.result[0].start_year) {
        req.result[0].start_year = req.old_result[0].start_year;
    }
    if (req.old_result[0].end_year > req.result[0].end_year) {
        req.result[0].end_year = req.old_result[0].old_result;
    }
    next();
}, query, response('range'));

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'city',
    table: 'municipio',
    tableField: 'nome',
    resultField: 'city_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'municipio_id',
        table: 'adh_idh'
    },
    join: {
        primary: 'id',
        foreign: 'municipio_id',
        foreignTable: 'adh_idh'
    }
}).addValue({
    name: 'state',
    table: 'estado',
    tableField: 'nome',
    resultField: 'state_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'estado_id',
        table: '@'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: '@'
    }
}).addValue({
    name: 'min_year',
    table: '@',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        table: '@',
        field: 'ano_censo'
    }
}).addValue({
    name: 'max_year',
    table: '@',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        table: '@',
        field: 'ano_censo'
    }
});

idhmrApp.get('/', rqf.parse(), (req, res, next) => {
  log.debug(req.sql.toParam());
  if(typeof req.filter === 'undefined' || Object.keys(req.filter).length === 0 ) {
      res.status(400);
      next({
          status: 400,
          message: 'Wrong/No filter specified'
      });
  }
  if ("state" in req.filter && !("city" in req.filter)) {
      req.sql.from('adh_idh_uf')
      .field('adh_idh_uf.idhm_r', 'total')
      .field('adh_idh_uf.ano_censo', 'year')
      .field('adh_idh_uf.estado_id', 'state_id');
  } else if ("city" in req.filter) {
      req.sql.from('adh_idh')
      .field('adh_idh.idhm_r', 'total')
      .field('adh_idh.ano_censo', 'year')
      .field('adh_idh.municipio_id', 'city_id')
      .field('adh_idh.estado_id', 'estado_id');
  } else {
        next({
            status: 400,
            message: 'Wrong/No filter specified'
        });
    }

  next();
}, rqf.build(),query, response('idhmr'));

module.exports = idhmrApp;
